@@ -9,19 +9,17 @@
 //    This file is part of the omniORB library
 //
 //    The omniORB library is free software; you can redistribute it and/or
-//    modify it under the terms of the GNU Library General Public
+//    modify it under the terms of the GNU Lesser General Public
 //    License as published by the Free Software Foundation; either
-//    version 2 of the License, or (at your option) any later version.
+//    version 2.1 of the License, or (at your option) any later version.
 //
 //    This library is distributed in the hope that it will be useful,
 //    but WITHOUT ANY WARRANTY; without even the implied warranty of
 //    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//    Library General Public License for more details.
+//    Lesser General Public License for more details.
 //
-//    You should have received a copy of the GNU Library General Public
-//    License along with this library; if not, write to the Free
-//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-//    02111-1307, USA
+//    You should have received a copy of the GNU Lesser General Public
+//    License along with this library. If not, see http://www.gnu.org/licenses/
 //
 //
 // Description:
@@ -48,9 +46,7 @@
 #include <poaimpl.h>
 #include <orbParameters.h>
 
-#ifdef HAVE_STD
 #include <memory>
-#endif
 
 OMNI_NAMESPACE_BEGIN(omni)
 
@@ -160,6 +156,9 @@ GIOP_S::dispatcher() {
     else if (pd_requestType == GIOP::CancelRequest) {
       return handleCancelRequest();
     }
+    else if (pd_requestType == GIOP::_NP_NoRequest) {
+      return handleNoRequest();
+    }
     else {
       if( omniORB::trace(1) ) {
 	omniORB::logger l;
@@ -175,7 +174,7 @@ GIOP_S::dispatcher() {
     return 0;
   }
 
-#ifndef HAS_Cplusplus_catch_exception_by_base
+#ifndef OMNI_HAS_Cplusplus_catch_exception_by_base
 #  define CATCH_AND_HANDLE(name) \
   catch(CORBA::name& ex) { \
     impl()->sendMsgErrorMessage(this, &ex); \
@@ -346,7 +345,7 @@ GIOP_S::handleRequest() {
     } \
 } while (0) 
 
-# ifndef HAS_Cplusplus_catch_exception_by_base
+# ifndef OMNI_HAS_Cplusplus_catch_exception_by_base
 
   // We have to catch each type of system exception separately
   // here to support compilers which cannot catch more derived
@@ -384,7 +383,6 @@ GIOP_S::handleRequest() {
     throw;
   }
 
-#ifdef HAVE_STD
   catch (const std::bad_alloc&) {
     // We keep logging as simple as possible to avoid too much allocation.
     omniORB::logs(1, "Error: upcall raised std::bad_alloc.");
@@ -409,7 +407,6 @@ GIOP_S::handleRequest() {
       impl()->sendSystemException(this,ex);
     }
   }
-#endif // HAVE_STD
 
   catch (const omni_thread_fatal& thr_ex) {
     if (omniORB::trace(1)) {
@@ -438,6 +435,8 @@ GIOP_S::handleRequest() {
     }
   }
   pd_state = ReplyCompleted;
+
+  clearValueTracker();
   clearDeadline();
   return 1;
 }
@@ -526,7 +525,7 @@ GIOP_S::handleLocateRequest() {
     } \
 } while (0) 
 
-# ifndef HAS_Cplusplus_catch_exception_by_base
+# ifndef OMNI_HAS_Cplusplus_catch_exception_by_base
 
   // We have to catch each type of system exception separately
   // here to support compilers which cannot catch more derived
@@ -545,6 +544,34 @@ GIOP_S::handleLocateRequest() {
     MARSHAL_SYSTEM_EXCEPTION();
   }
 
+  pd_state = ReplyCompleted;
+
+  clearValueTracker();
+  clearDeadline();
+  return 1;
+}
+
+
+////////////////////////////////////////////////////////////////////////
+CORBA::Boolean
+GIOP_S::handleNoRequest() {
+
+  // There is no request to handle. We set the state as if we have
+  // handled a request, so we can wait for the next one.
+
+  CORBA::Boolean data_in_buffer = 0;
+  if (pd_rdlocked) {
+    // This is the thread that is reading from the connection. We
+    // check if we have previously queued giopStream_Buffers on the
+    // connection. In other words, we might have previously read
+    // too much stuff out of the connection and these data belong to
+    // other requests. If that is the case, we notify the giopServer
+    // that there are already buffers waiting to be read.
+    giopStrand& s = strand();
+    data_in_buffer = ((s.head) ? 1 : 0);
+  }
+  pd_worker->server()->notifyWkPreUpCall(pd_worker, data_in_buffer);
+
   pd_state = ReplyCompleted;
   clearDeadline();
   return 1;
