@@ -155,6 +155,11 @@ giopImpl12::inputQueueMessage(giopStream* g, giopStream_Buffer* b) {
   CORBA::ULong reqid;
 
   if (mtype != GIOP::CloseConnection && mtype != GIOP::MessageError) {
+    if (b->size < 16) {
+      giopStream_Buffer::deleteBuffer(b);
+      OMNIORB_THROW(MARSHAL, MARSHAL_PassEndOfMessage,
+                    (CORBA::CompletionStatus)g->completion());
+    }
 
     // unmarshal request id.
     reqid = *((CORBA::ULong*)(hdr + 12));
@@ -519,6 +524,7 @@ giopImpl12::inputNewFragment(giopStream* g) {
 
   g->pd_inb_mkr = (void*)(hdr + 16);
   g->pd_inb_end = (void*)g->inputBufferLast();
+  OMNIORB_ASSERT(g->pd_inb_mkr <= g->pd_inb_end);
 
   g->inputExpectAnotherFragment(((hdr[6] & 0x2) ? 1 : 0));
 
@@ -602,6 +608,7 @@ giopImpl12::inputReplyBegin(giopStream* g,
 
   g->pd_inb_mkr = (void*)(hdr + 16);
   g->pd_inb_end = (void*)g->inputBufferLast();
+  OMNIORB_ASSERT(g->pd_inb_mkr <= g->pd_inb_end);
 
   g->inputExpectAnotherFragment(((hdr[6] & 0x2) ? 1 : 0));
 
@@ -644,7 +651,16 @@ giopImpl12::inputMessageBegin(giopStream* g,
     }
     else {
       inputNewServerMessage(g);
-      goto again;
+
+      if (!g->pd_currentInputBuffer) {
+        if (g->pd_input)
+          goto again;
+
+        // inputNewServerMessage queued a message for another thread, so
+        // there is no request message for this thread to handle at present.
+        ((GIOP_S*)g)->requestType(GIOP::_NP_NoRequest);
+        return;
+      }
     }
   }
 
