@@ -53,6 +53,14 @@ const char*    sslContext::certificate_authority_path  = 0;
 const char*    sslContext::key_file                    = 0;
 const char*    sslContext::key_file_password           = 0;
 const char*    sslContext::cipher_list                 = 0;
+const char*    sslContext::ciphersuites                = 0;
+
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L
+int            sslContext::min_proto_version           = TLS1_2_VERSION;
+#else
+int            sslContext::min_proto_version           = 0;
+#endif
+int            sslContext::max_proto_version           = 0;
 int            sslContext::verify_mode                 =
                             (SSL_VERIFY_PEER | SSL_VERIFY_FAIL_IF_NO_PEER_CERT);
 int            sslContext::verify_mode_incoming        = -1;
@@ -277,9 +285,22 @@ sslContext::set_method() {
 /////////////////////////////////////////////////////////////////////////
 void
 sslContext::set_supported_versions() {
+
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L
+  if (!SSL_CTX_set_min_proto_version(pd_ctx, sslContext::min_proto_version)) {
+    report_error();
+    OMNIORB_THROW(INITIALIZE, INITIALIZE_TransportError, CORBA::COMPLETED_NO);
+  }
+
+  if (!SSL_CTX_set_max_proto_version(pd_ctx, sslContext::max_proto_version)) {
+    report_error();
+    OMNIORB_THROW(INITIALIZE, INITIALIZE_TransportError, CORBA::COMPLETED_NO);
+  }
+#else
   SSL_CTX_set_options(pd_ctx,
                       SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3 |
                       SSL_OP_NO_TLSv1 | SSL_OP_NO_TLSv1_1);
+#endif
 }
 
 /////////////////////////////////////////////////////////////////////////
@@ -346,6 +367,14 @@ sslContext::set_cipher_list() {
     report_error();
     OMNIORB_THROW(INITIALIZE, INITIALIZE_TransportError, CORBA::COMPLETED_NO);
   }
+
+#if OPENSSL_VERSION_NUMBER >= 0x10101000L
+  if (sslContext::ciphersuites &&
+      !SSL_CTX_set_ciphersuites(pd_ctx, sslContext::ciphersuites)) {
+    report_error();
+    OMNIORB_THROW(INITIALIZE, INITIALIZE_TransportError, CORBA::COMPLETED_NO);
+  }
+#endif
 }
 
 /////////////////////////////////////////////////////////////////////////
@@ -386,6 +415,13 @@ sslContext::set_privatekey() {
 void
 sslContext::set_DH() {
 
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
+
+  // OpenSSL 3 recommends using the "auto" DH parameters
+  SSL_CTX_set_dh_auto(pd_ctx, 1);
+
+#else
+  
   DH* dh = DH_new();
   if (dh == 0) {
     OMNIORB_THROW(INITIALIZE, INITIALIZE_TransportError, CORBA::COMPLETED_NO);
@@ -439,6 +475,8 @@ sslContext::set_DH() {
   
   SSL_CTX_set_tmp_dh(pd_ctx, dh);
   DH_free(dh);
+
+#endif  // OpenSSL versions before 3
 }
 
 /////////////////////////////////////////////////////////////////////////
