@@ -7,19 +7,17 @@
 //    This file is part of the omnithread library
 //
 //    The omnithread library is free software; you can redistribute it and/or
-//    modify it under the terms of the GNU Library General Public
+//    modify it under the terms of the GNU Lesser General Public
 //    License as published by the Free Software Foundation; either
-//    version 2 of the License, or (at your option) any later version.
+//    version 2.1 of the License, or (at your option) any later version.
 //
 //    This library is distributed in the hope that it will be useful,
 //    but WITHOUT ANY WARRANTY; without even the implied warranty of
 //    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//    Library General Public License for more details.
+//    Lesser General Public License for more details.
 //
-//    You should have received a copy of the GNU Library General Public
-//    License along with this library; if not, write to the Free
-//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
-//    02111-1307, USA
+//    You should have received a copy of the GNU Lesser General Public
+//    License along with this library. If not, see http://www.gnu.org/licenses/
 //
 
 //
@@ -766,20 +764,25 @@ omni_thread::exit(void* return_value)
 	DB(cerr << "omni_thread::exit: thread " << me->id() << " detached "
 	   << me->detached << " return value " << return_value << endl);
 
-	if (me->_values) {
-	  for (key_t i=0; i < me->_value_alloc; i++) {
-	    if (me->_values[i]) {
-	      delete me->_values[i];
-	    }
-	  }
-	  delete [] me->_values;
-	  me->_values = 0;
-	}
-
 	if (me->detached) {
 	  delete me;
-	} else {
+	}
+	else {
 	  me->return_val = return_value;
+
+	  if (me->_values) {
+	    // Delete per-thread state here, to ensure value destructors
+	    // are executed by this thread.
+	  
+	    for (key_t i=0; i < me->_value_alloc; i++) {
+	      if (me->_values[i]) {
+		delete me->_values[i];
+	      }
+	    }
+	    delete [] me->_values;
+	    me->_values = 0;
+	    me->_value_alloc = 0;
+	  }
 	}
       }
     else
@@ -809,12 +812,19 @@ omni_thread::self(void)
     me = TlsGetValue(self_tls_index);
 
     if (me == 0) {
-      DB(cerr << "omni_thread::self: called with a non-ominthread. NULL is returned." << endl);
+      DB(cerr << "omni_thread::self: called with a non-omnithread. NULL is returned." << endl);
     }
     return (omni_thread*)me;
 }
 
 
+unsigned long
+omni_thread::plat_id()
+{
+    return GetCurrentThreadId();
+}
+
+
 void
 omni_thread::yield(void)
 {
@@ -931,7 +941,6 @@ public:
   }
   inline ~omni_thread_dummy()
   {
-    TlsSetValue(self_tls_index, (LPVOID)0);
   }
 };
 
@@ -951,10 +960,19 @@ omni_thread::release_dummy()
   if (!self || !self->_dummy)
     throw omni_thread_invalid();
 
+  TlsSetValue(self_tls_index, (LPVOID)0);
+
   omni_thread_dummy* dummy = (omni_thread_dummy*)self;
   delete dummy;
 }
 
+void
+omni_thread::release_finished_dummy(omni_thread* d)
+{
+  omni_thread_dummy* dummy = (omni_thread_dummy*)d;
+  delete dummy;
+}
+
 
 #if defined(__DMC__) && defined(_WINDLL)
 BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
