@@ -100,6 +100,14 @@
 #define DB(x) // x
 //#include <iostream.h> or #include <iostream> if DB is on.
 
+
+// If PthreadDraftVersion is not defined, assume it is modern
+
+#ifndef PthreadDraftVersion
+#  define PthreadDraftVersion 10
+#endif
+
+
 #if (PthreadDraftVersion <= 6)
 #define ERRNO(x) (((x) != 0) ? (errno) : 0)
 #ifdef __VMS
@@ -988,7 +996,6 @@ public:
   }
   inline ~omni_thread_dummy()
   {
-    pthread_setspecific(self_key, 0);
   }
 };
 
@@ -1008,6 +1015,15 @@ omni_thread::release_dummy()
   if (!self || !self->_dummy)
     throw omni_thread_invalid();
 
+  pthread_setspecific(self_key, 0);
+
   omni_thread_dummy* dummy = (omni_thread_dummy*)self;
   delete dummy;
 }
+
+void
+omni_thread::release_finished_dummy(omni_thread* d)
+{
+  omni_thread_dummy* dummy = (omni_thread_dummy*)d;
+  delete dummy;
+}
